<?php
namespace boardtalk\smartpreview\controller;

use phpbb\template\template;
use phpbb\controller\helper;
use phpbb\db\driver\driver_interface;
use phpbb\user;

class main_controller
{
    protected $template;
    protected $helper;
    protected $db;
    protected $user;

    public function __construct(template $template, helper $helper, driver_interface $db, user $user)
    {
        $this->template = $template;
        $this->helper   = $helper;
        $this->db       = $db;
        $this->user     = $user;
    }

    public function handle()
    {
        // ── DEBUGGING ERRORS ──
        ini_set('display_errors', 1);
        ini_set('display_startup_errors', 1);
        error_reporting(E_ALL);

        // 1) Fetch latest topics
        $limit = 15;
        $sql = 'SELECT t.topic_id, t.topic_title, t.topic_first_post_id, t.topic_last_post_id, t.topic_views
                FROM ' . TOPICS_TABLE . ' t
                WHERE t.topic_visibility = 1
                ORDER BY t.topic_last_post_time DESC';
        $res = $this->db->sql_query_limit($sql, $limit);

        $topics   = [];
        $post_ids = [];
        while ($row = $this->db->sql_fetchrow($res))
        {
            $tid = (int) $row['topic_id'];
            $topics[$tid] = [
                'title'    => $row['topic_title'],
                'first_id' => (int) $row['topic_first_post_id'],
                'last_id'  => (int) $row['topic_last_post_id'],
                'views'    => (int) $row['topic_views'],
                'replies'  => 0,
                // placeholders
                'first_user'   => '',
                'first_text'   => '',
                'first_time'   => 0,
                'first_avatar' => '',
                'last_user'    => '',
                'last_text'    => '',
                'last_time'    => 0,
            ];

            $post_ids[] = $row['topic_first_post_id'];
            if ($row['topic_last_post_id'] !== $row['topic_first_post_id'])
            {
                $post_ids[] = $row['topic_last_post_id'];
            }
        }
        $this->db->sql_freeresult($res);

        // 2) Count replies
        $topic_ids = array_keys($topics);
        if (!empty($topic_ids))
        {
            $sql = 'SELECT topic_id, COUNT(post_id) AS total_posts
                    FROM ' . POSTS_TABLE . '
                    WHERE ' . $this->db->sql_in_set('topic_id', $topic_ids) . '
                    GROUP BY topic_id';
            $r2 = $this->db->sql_query($sql);
            while ($row2 = $this->db->sql_fetchrow($r2))
            {
                $tid2 = (int) $row2['topic_id'];
                $topics[$tid2]['replies'] = max(0, (int) $row2['total_posts'] - 1);
            }
            $this->db->sql_freeresult($r2);
        }

        // 3) Fetch posts + avatars
        if (!empty($post_ids))
        {
            $ids_sql = $this->db->sql_in_set('p.post_id', array_unique($post_ids));
            $sql = 'SELECT p.post_id, p.post_text, p.post_time,
                           u.username, u.user_avatar, u.user_avatar_type,
                           u.user_avatar_width, u.user_avatar_height
                    FROM ' . POSTS_TABLE . ' p
                    LEFT JOIN ' . USERS_TABLE . ' u
                      ON p.poster_id = u.user_id
                    WHERE ' . $ids_sql;
            $r = $this->db->sql_query($sql);

            while ($row = $this->db->sql_fetchrow($r))
            {
                $pid = (int) $row['post_id'];
                foreach ($topics as $tid => &$t)
                {
                    if ($pid === $t['first_id'])
                    {
                        $t['first_user']   = $row['username'];
                        $t['first_text']   = $row['post_text'];
                        $t['first_time']   = (int) $row['post_time'];
                        $t['first_avatar'] = get_user_avatar(
                            $row['user_avatar'],
                            $row['user_avatar_type'],
                            $row['user_avatar_width'],
                            $row['user_avatar_height'],
                            $row['username']
                        );
                    }
                    if ($pid === $t['last_id'])
                    {
                        $t['last_user'] = $row['username'];
                        $t['last_text'] = $row['post_text'];
                        $t['last_time'] = (int) $row['post_time'];
                    }
                }
                unset($t);
            }
            $this->db->sql_freeresult($r);
        }

		// 4) Assign to template
		foreach ($topics as $tid => $t)
		{
			// FIRST post excerpt (30 words)
			$words_first       = preg_split('/\s+/', strip_tags($t['first_text']), -1, PREG_SPLIT_NO_EMPTY);
			$first_truncated   = count($words_first) > 30;
			$first_excerpt     = $first_truncated
							   ? implode(' ', array_slice($words_first, 0, 30)) . '...'
							   : implode(' ', $words_first);

			// LAST post excerpt (30 words)
			$words_last        = preg_split('/\s+/', strip_tags($t['last_text']), -1, PREG_SPLIT_NO_EMPTY);
			$last_truncated    = count($words_last) > 30;
			$last_excerpt      = $last_truncated
							   ? implode(' ', array_slice($words_last, 0, 30)) . '...'
							   : implode(' ', $words_last);

			$this->template->assign_block_vars('topics', [
				'HAS_AVATAR'         => (bool) $t['first_avatar'],
				'AVATAR'             => $t['first_avatar'],

				'TOPIC_TITLE'        => $t['title'],
				'U_FIRST_POST'       => append_sid('viewtopic.php', 't=' . $tid),
				'FIRST_POSTER_FULL'  => $t['first_user'],
				'FIRST_POST_TIME'    => date('j M Y, H:i', $t['first_time']),
				'FIRST_POST_EXCERPT' => $first_excerpt,
				'FIRST_TRUNCATED'    => (int) $first_truncated,

				'TOPIC_VIEWS'        => $t['views'],
				'TOPIC_REPLIES'      => $t['replies'],

				'U_LAST_POST'        => append_sid('viewtopic.php', 'p=' . $t['last_id']) . '#p' . $t['last_id'],
				'LAST_POSTER_FULL'   => $t['last_user'],
				'LAST_POST_TIME'     => date('j M Y, H:i', $t['last_time']),
				'LAST_POST_EXCERPT'  => $last_excerpt,
				'LAST_TRUNCATED'     => (int) $last_truncated,
			]);
		}

        return $this->helper->render(
            'recent_body.html',
            $this->user->lang('SMARTPREVIEW_TITLE')
        );
    }
}
